package j51.diseqc;

import java.awt.*;
import javax.swing.*;

import j51.intel.*;
import j51.swing.*;
import j51.util.Hex;

abstract class Atmel24C16 implements MCS51Peripheral
{
	private boolean oldSCL,oldSDA;
	private int eeprom[] = new int[16384];
	private boolean read = false;
	private boolean ack = false;
	private int status = 0;
	private int bitCount;
	private int byteCount;
	private int addr;
	private int cmd;
	private int value;
	protected MCS51 cpu;
	
	public Atmel24C16()
	{
		stopCondition();
	}

	void stopCondition()
	{
		bitCount = 0;
		byteCount = 0;
		read = false;
		ack = false;
		
	}


	void startCondition()
	{
		stopCondition();
	}

	private void received(int v)
	{
		byteCount++;
		if (byteCount == 1)
		{
			cmd = v;
			if ((cmd & 1) != 0)
			{
				//System.out.println("Read "+Hex.bin2word(addr)+" = "+Hex.bin2byte(eeprom[addr]));
				read = true;
				value = eeprom[addr++];
			}

			return;
		}

		// Write
		if ((cmd & 1) == 0)
		{
			if (byteCount == 2)
			{
				addr = (((cmd << 7) & 0x700) | v) ;
				return;
			}

			//System.out.print("W "+Hex.bin2word(addr)+" = "+Hex.bin2byte(value)+" ");
			eeprom[addr++] = v;
		}


	}

	final void writePort(boolean currentSCL,boolean currentSDA)
	{
		if (!currentSCL && ack)
		{
			ack = false;
		}

		if (ack)
		{
			SDA(false);
		}

		if (currentSCL && oldSCL && !oldSDA && currentSDA && !ack)
		{
			stopCondition();
			oldSDA = currentSDA;
			return;
		}

		if (currentSCL && oldSCL && oldSDA && !currentSDA)
		{
			startCondition();
			oldSDA = currentSDA;
		}


		// Check clock
		if (currentSCL && !oldSCL)
		{
			if (read)
			{
				if (++bitCount == 9)
				{
					//System.out.println("Read "+Hex.bin2word(addr)+" = "+Hex.bin2byte(eeprom[addr]));


					value = eeprom[addr++];
					bitCount = 0;
				}
				else
				{
					//System.out.println("Write bit "+bitCount+((value & 0x80) != 0));
					if ((value & 0x80) != 0)
					{
						SDA(true);
					}
					else
					{
						SDA(false);
					}
					value <<= 1;
				}
			}
			else
			{
				if (++bitCount == 9)
				{
					//System.out.println("start ack");
					received(value & 0xff);
					SDA(false);
					bitCount = 0;
					ack = true;
				}
				else
				{
					//System.out.println("read Bit "+bitCount+" "+ currentSDA);
					value <<= 1;
					if (currentSDA)
						value |= 1;
					else
						value &= ~1;
				}
			}


		}

		oldSDA = currentSDA;
		oldSCL = currentSCL;
	}

	abstract protected void SDA(boolean value);
	

	public void registerCpu(MCS51 cpu)
	{
		this.cpu = cpu;
	}
}


class DiseqcAtmel24C16 extends Atmel24C16 implements SfrWriteListener
{
	private final int SCL = 0x40;
	private final int SDA = 0x80;
	
	public void registerCpu(MCS51 cpu)
	{
		super.registerCpu(cpu);
		cpu.addSfrWriteListener(MCS51Constants.P0,this);

	}

	protected void SDA(boolean sda)
	{
		if (sda)
			cpu.sfrSet(MCS51Constants.P0,SDA);
		else
			cpu.sfrReset(MCS51Constants.P0,SDA);
	}

	public void sfrWrite(int reg,int value)
	{
		writePort((value & SCL) != 0,(value & SDA) != 0);
	}
}


class JLed extends JPanel
{
	
	private JTextField led = new JTextField(6);
	private boolean mode = true;
	private Color color;
	
	public JLed(String label,Color color)
	{
		JFactory.setTitle(this,label);
		add(led);
		set(false);
		led.setEditable(false);
		this.color = color;
	}

	public JLed(String label)
	{
		this(label,Color.red);
	}
	
	void set(boolean mode)
	{
		if (mode != this.mode)
		{
			this.mode = mode;
			//led.setText(mode ? "On" : "Off");
			//led.setForeground(mode ? JFactory.getColorSelected() :
			//JFactory.getColorNormal());
			led.setBackground(mode ? color : color.black);
		}
	}

	boolean get()
	{
		return mode;
	}
	
	void on()
	{
		set(false);
	}
		
}

class JMotor extends JPanel
{
	int value = -1;
	JTextField f;
	JMotor(int min,int max,int value)
	{
		f = new JTextField();
		Font font = f.getFont();
		font = new Font("Monospaced",font.BOLD,font.getSize()*4);
		f.setFont(font);
		f.setForeground(Color.green);
		f.setBackground(Color.black);
		f.setEditable(false);
		setValue(0);
		add(f);
		JFactory.setTitle(this,"Position");

	}

	public void setValue(int value)
	{
		if (value == this.value)
			return;
		this.value = value;
		String s;
		
		if (value == 0)
			s = "  00  ";
		else
		{
			String p;
			if (value < 0)
			{
				p = "E";
				value = - value;
			}
			else
			{
				p = "W";
			}
			int i = value / 10;
			int d = value % 10;
			s = i+"."+d+p;
		}

		while (s.length() < 6)
			s = " "+s;
		f.setText(s);
	}
}

class DiseqcPeripheral extends JPanel implements MCS51Peripheral,
	  SfrWriteListener,SfrReadListener,MachineCycles
{
	private final int MIN = -900;
	private final int MAX =  900;
	//private int status = 0;
	private MCS51 cpu;
	private JLed ledw = new JLed("West",Color.green);
	private JLed lede = new JLed("East",Color.green);
	private JLed led  = new JLed("Status");
	private JLed ledp = new JLed("Power",Color.yellow);
	private JToggleButton gow = new JToggleButton("GO WEST");
	private JToggleButton goe = new JToggleButton("GO EAST");
	private JMotor position = new JMotor(MIN,MAX,0);
	private int motorPosition = 0;
	private int pulseWidth,pulseTime;
	boolean motorPower;
	private int motorCount;
	private int motorStatus;
	
	public DiseqcPeripheral()
	{
		super(new GridBagLayout());
		JFactory.setTitle(this,"Diseqc on LPC764");
		
		GridBagConstraints g = new GridBagConstraints();
		g.gridx = 0; g.gridy = 0;g.gridwidth = 1;g.gridheight = 1;
		g.anchor = g.CENTER; g.fill = g.NONE; g.insets=new Insets(2,2,2,2);

		
		/*position.setMinorTickSpacing(2);
		position.setMajorTickSpacing(30);
		position.setPaintTicks(true);
		position.setPaintLabels(true);*/
		
		g.gridx = 0;g.gridwidth = 4;
		add(position,g);
		
		g.gridy++;g.gridx=0;g.gridwidth = 1;
		add(lede,g);
		g.gridx += 1;
		add(led,g);
		g.gridx += 1;
		add(ledp,g);
		g.gridx += 1;
		add(ledw,g);

		g.gridy++;g.gridx = 0;g.gridwidth = 2;
		add(goe,g);
		g.gridx += 2;
		add(gow,g);
		
		
	}

	
	public int sfrRead(int reg)
	{
		int value = cpu.sfr(reg);
		value |= 3;

		if (goe.isSelected())
			value &= ~1;
		if (gow.isSelected())
			value &= ~2;

		return value;
	}

	public void writeP1(int value)
	{
		lede.set((value & 0x80) != 0);
		ledw.set((value & 0x40) != 0);
	}

	public void sfrWrite(int reg,int value)
	{
		if (reg == MCS51Constants.P0)
		{
			led.set((value & 4) == 0);
			ledp.set((value & 8) == 0);

			motorPower = ledp.get();
			
			if (!motorPower)
			{
				motorCount = pulseWidth;
				motorStatus = 0;
			}

		}
		else
		{
			lede.set((value & 0x80) != 0);
			ledw.set((value & 0x40) != 0);
		}
			
	}

	public void cycles(int n)
	{
		if (!motorPower)
			return ;
		motorCount -= n;
		if (motorCount <= 0)
		{
			if (motorStatus == 0)
			{
				int lastPosition = motorPosition;
				if (lede.get())
				{
					if (motorPosition > MIN)
						motorPosition --;
				}
				else
				{
					if (motorPosition < MAX)
						motorPosition++;
				}

				motorCount = pulseTime;

				if (lastPosition != motorPosition)
				{
					motorStatus++;
					position.setValue(motorPosition);
					int p1 = cpu.sfr(MCS51Constants.P1);
					p1 |= 0x10;
					cpu.sfr(MCS51Constants.P1,p1);
				}
			}
			else
			{
				position.setValue(motorPosition);
				int p1 = cpu.sfr(MCS51Constants.P1);
				p1 &= ~0x10;
				cpu.sfr(MCS51Constants.P1,p1);
				motorStatus = 0;
				motorCount = pulseTime;
			}
	
		}				  
	}


	public void registerCpu(MCS51 cpu)
	{
		this.cpu = cpu;
		cpu.addSfrWriteListener(MCS51Constants.P0,this);
		cpu.addSfrWriteListener(MCS51Constants.P1,this);
		cpu.addSfrReadListener(MCS51Constants.P0,this);
		cpu.addMachineCycle(this);

		pulseWidth = (int)(cpu.oscillator() / cpu.cycles() / 1000) * 12;
		pulseTime = (int)(cpu.oscillator() / cpu.cycles() / 1000) * 20;
	}
}

public class Diseqc extends MCS51
{
	public Diseqc()
	{
		oscillator = 11059200;
		machineCycles = 6;

		addPeripheral(new j51.intel.Timer());
		addPeripheral(new JUart());
		addPeripheral(new DiseqcPeripheral());
		addPeripheral(new DiseqcAtmel24C16());
	}
}
